@@ -66,9 +66,16 @@ class PolicyKitService(dbus.service.Object):
                 kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit')
                 pid = dbus.UInt32(os.getpid())
 
-                if 'yes' != kit.IsProcessAuthorized(action, pid, False):
+                granted = kit.IsProcessAuthorized(action, pid, False)
+                logging.info('process authorization: %r', granted)
+
+                if 'no' == granted:
                     raise AccessDeniedException('Process not authorized by PolicyKit')
-                if 'yes' != kit.IsSystemBusNameAuthorized(action, sender, False):
+
+                granted = kit.IsSystemBusNameAuthorized(action, sender, False)
+                logging.info('authorizatoin of system bus name: %r', granted)
+
+                if 'no' == granted:
                     raise AccessDeniedException('Session not authorized by PolicyKit')
 
         except AccessDeniedException:
@@ -80,7 +87,7 @@ class PolicyKitService(dbus.service.Object):
 class ExternalToolDriver(PolicyKitService):
     '''
     A D-BUS service which mainly is implemented
-    as wrapper arround some external program.
+    as wrapper around some external program.
     '''
 
     # pylint: disable-msg=C0103,E0602
@@ -344,13 +351,16 @@ class DetectParametersDriver(IrRecordDriver):
             return
 
         if line.startswith(self._token_random_buttons):
+            # TODO: The talk of "steps" here does not make much sense
+            # in terms of a GTK+ progress bar.
+
             self.RequestAction(
                 _('Press random buttons on your remote control.'),
-                _('It is very important that you press many different ' +
-                  'buttons and hold them down for approximately one ' +
-                  'second. Each button should move the progress bar ' +
-                  'by at least one, but in no case by more than ten ' +
-                  'steps.'))
+                _('When you press the Start button, it is very important '
+                  'that you press many different buttons and hold them down ' +
+                  'for approximately one second. Each button should move the ' +
+                  'progress bar by at least one step, but in no case by more ' +
+                  'than ten steps.'))
 
             return
 
@@ -520,7 +530,7 @@ class BackendService(PolicyKitService):
         if self.connection.list_exported_child_objects('/'):
             return True
 
-        print 'Terminating %s due inactivity.' % self.SERVICE_NAME
+        print 'Terminating %s due to inactivity.' % self.SERVICE_NAME
         self.__loop.quit()
 
         return False
@@ -554,41 +564,37 @@ class BackendService(PolicyKitService):
         output = file(newfile, 'w')
         for line in file(oldfile, 'r'):
 
-            # Deal with lines starting with REMOTE_,
-            # replacing their values with ours:
-
+            # Identify directives starting with REMOTE_ and replacing their values with ours.
+            # Remove entry from the dict on match, so we know what was written.
             match = self.__re_remote_directive.match(line)
+            value = match and remote_values.pop(match.group(1), None) or None
 
-            if match:
-                # Remove entry from the dict, so we know what was written.
-                value = remote_values.pop(match.group(1), None)
-
-                if value is not None:
-                    print >> output, ('%s"%s"' % (match.group(0), value))
-                    continue
-
-            # Deal with lines starting with RECEIVER_,
-            # replacing their values with ours:
+            if value is not None:
+                print >> output, ('%s"%s"' % (match.group(0), value))
+                continue
 
+            # Identify directives starting with RECEIVER_ and replacing their values with ours.
+            # Remove entry from the dict on match, so we know what was written.
             match = self.__re_receiver_directive.match(line)
+            value = match and receiver_values.pop(match.group(1), None) or None
 
-            if match:
-                # Remove entry from the dict, so we know what was written.
-                value = receiver_values.pop(match.group(1), None)
-
-                if value is not None:
-                    print >> output, ('%s"%s"' % (match.group(0), value))
-                    continue
+            if value is not None:
+                print >> output, ('%s"%s"' % (match.group(0), value))
+                continue
 
             # Deal with the START_LIRCD line:
 
             match = self.__re_start_lircd.match(line)
 
             if match:
-                #TODO: pychecker says "gnome_lirc_properties/backend.py:588: Using a conditional statement with a constant value (true)"
-                value = (start_lircd is None and 'true') or start_lircd
-                print >> output, (match.group(0) + value)
+                # pychecker says "Using a conditional statement with a constant value (true)",
+                # which is ridicilous, considering Python 2.4 doesn't have conditional statements
+                # yet (PEP 308, aka. 'true_value if condition else false_value') and the expression
+                # below ('condition and true_value or false_value') is the recommended aquivalent.
+                value = (start_lircd is None) and 'true' or start_lircd
                 start_lircd = None
+
+                print >> output, (match.group(0) + value)
                 continue
 
             output.write(line)
@@ -826,6 +832,13 @@ def get_service(bus=None):
     return service
 
 if __name__ == '__main__':
+    # Support full tracing when --debug switch is passed:
+
+    from sys import argv
+
+    if '--debug' in argv or '-d' in argv:
+        logging.getLogger().setLevel(logging.NOTSET)
+
     # Integrate DBus with GLib main loops.
 
     from dbus.mainloop.glib import DBusGMainLoop
