@@ -19,11 +19,12 @@
 LIRC specific classes.
 '''
 
-import errno, gobject, logging, os, re
+import errno, gobject, logging, os, re, tarfile
 
 from datetime              import datetime
 from gettext               import gettext as _
 from gnome_lirc_properties import config
+from locale                import LC_TIME, setlocale
 from StringIO              import StringIO
 from socket                import AF_UNIX, socket, error as SocketError
 
@@ -60,7 +61,6 @@ class Receiver(object):
         and return the first one found as "supplied remote".
         '''
 
-        # pylint: disable-msg=W0141
         remotes = self.__compatible_remotes or []
         remotes = map(remotes_db.get, remotes)
         remotes = filter(None, remotes)
@@ -81,6 +81,9 @@ class Receiver(object):
     def __repr__(self):
         return '<Receiver: vendor=%s, product=%s>' % (self.__vendor, self.__product)
 
+    def __getitem__(self, key):
+        return self.__properties[key]
+
     # pylint: disable-msg=W0212
     vendor = property(lambda self: self.__vendor)
     vendor_id = property(lambda self: self.__vendor_id)
@@ -88,6 +91,7 @@ class Receiver(object):
     product = property(lambda self: self.__product)
     product_id = property(lambda self: self.__product_id)
 
+    device = property(lambda self: self.__properties.get('device'))
     device_nodes = property(lambda self: self.__properties.get('device-nodes'))
     kernel_module = property(lambda self: self.__properties.get('kernel-module'))
     lirc_driver = property(lambda self: self.__properties.get('lirc-driver', 'default'))
@@ -100,13 +104,15 @@ class Remote(object):
 
     # pylint: disable-msg=R0913
 
-    def __init__(self, filename = None, vendor = None, product = None,
-                 properties = list(), key_codes = dict(), contributor = None):
+    def __init__(self, database = None, filename = None,
+                 vendor = None, product = None, contributor = None,
+                 properties = list(), key_codes = dict()):
 
         self.__properties = dict(properties)
         self.__property_order = [item[0] for item in properties]
         self.__key_codes = key_codes
         self.__filename = filename
+        self.__database = database
 
         self.__vendor = vendor
         self.__product = product
@@ -118,10 +124,12 @@ class Remote(object):
         Writes configuration file of this remote to writer.
         '''
 
-        # Write comment header with meta information
-
+        # Retrieve locale independent timestamp:
+        locale = setlocale(LC_TIME, 'C')
         now = datetime.now().strftime('%c')
+        setlocale(LC_TIME, locale)
 
+        # Write comment header with meta information:
         print >> writer, '# LIRC configuration file for %s' % self.name
         print >> writer, '# Generated by GNOME LIRC properties on %s' % now
         print >> writer, '# from %s' % self.filename
@@ -171,7 +179,6 @@ class Remote(object):
         '''
         Retreives the configuration file of this remote as string.
         '''
-
         return self.write(StringIO()).getvalue()
 
     def __set_properties(self, value):
@@ -200,6 +207,8 @@ class Remote(object):
     contributor = property(lambda self: self.__contributor)
 
     configuration = property(__get_configuration)
+
+    database = property(lambda self: self.__database)
     filename = property(lambda self: self.__filename)
     key_codes = property(lambda self: self.__key_codes)
     properties = property(lambda self: self.__properties, __set_properties)
@@ -214,31 +223,82 @@ class RemotesDatabase(object):
         super(RemotesDatabase, self).__init__()
         self.__remotes = dict()
 
-    def load_folder(self, path = config.LIRC_REMOTES_DATABASE):
+    def clear(self):
+        '''
+        Removes all database entries.
+        '''
+
+        self.__remotes.clear()
+
+    def load_folder(self, root=config.LIRC_REMOTES_DATABASE):
         '''
         Recursively reads all configuration files found in that folder.
         '''
 
-        logging.info('Reading remote database from %s...', path)
+        if not os.path.isdir(root):
+            logging.warning('Cannot read remote database folder: %s', root)
+            return False
 
-        for path, subdirs, files in os.walk(path):
+        logging.info('Reading remote database from %s...', root)
+
+        for path, subdirs, files in os.walk(root):
             # skip folders with meta information
             subdirs[:] = [name for name in subdirs
                           if not name in ('CVS', ) and
                              not name.startswith('.')]
 
+            # extract path component after the root path for current folder:
+            relative_path = path[len(root):]
+
+            while os.path.isabs(relative_path):
+                relative_path = relative_path[len(os.path.sep):]
+
+            # read files in current folder:
             for name in files:
                 if not name.startswith('.'):
-                    self.load(os.path.join(path, name))
+                    self.read(reader=open(os.path.join(path, name)),
+                              filename=os.path.join(relative_path, name),
+                              database=root)
+
+        return True
+
+    def load_tarball(self, path=config.LIRC_REMOTES_TARBALL):
+        '''
+        Reads remote configurations from tarball.
+        '''
+
+        if not os.path.isfile(path):
+            return False
+
+        if not tarfile.is_tarfile(path):
+            logging.warning('Bad remote control archive: %s', path)
+            return False
+
+        logging.info('Reading remote database from %s...', path)
+        tarball = tarfile.open(path)
+
+        for entry in tarball:
+            if not entry.isfile():
+                continue
+            if os.path.basename(entry.name) == 'README':
+                continue
+
+            self.read(reader=tarball.extractfile(entry.name),
+                      filename=entry.name, database=path,
+                      replace=True)
+
+        return True
 
     def load(self, filename):
         '''
         Reads remote configuration from filename.
         '''
 
-        self.read(open(filename))
+        self.read(open(filename),
+                  database=os.path.dirname(filename),
+                  filename=os.path.basename(filename))
 
-    def read(self, reader):
+    def read(self, reader, filename=None, database=None, replace=False):
         '''
         Reads remote configuration from reader.
         '''
@@ -246,30 +306,31 @@ class RemotesDatabase(object):
         parser = RemotesParser()
 
         try:
-            parser.read(reader)
+            parser.read(reader, filename, database)
 
         except ParseError, ex:
             logging.error(ex.message)
             return False
 
-        self.__update_remotes(parser.remotes)
+        self.__update_remotes(parser.remotes, replace)
 
         return True
 
-    def __update_remotes(self, remotes):
+    def __update_remotes(self, remotes, replace=False):
         '''
         Updates the remotes dictionary.
         '''
 
         for remote in remotes:
-            if self.__remotes.has_key(remote.name):
-                logging.warning('Remote with name %s listed twice in %s and %s.',
-                                remote.name, self.__remotes[remote.name].filename,
-                                remote.filename)
+            previous_remote = self.__remotes.get(remote.name)
 
+            if not previous_remote or (replace and remote.database != previous_remote.database):
+                self.__remotes[remote.name] = remote
                 continue
 
-            self.__remotes[remote.name] = remote
+            logging.warning('%s: Remote %s listed twice in %s and %s.',
+                            remote.database, remote.name, remote.filename,
+                            previous_remote.filename)
 
     def get(self, name, default=None):
         '''
@@ -302,6 +363,7 @@ class RemotesParser(object):
 
         self.__current_line = 0
         self.__filename = None
+        self.__database = None
 
         self.__next_remote()
 
@@ -355,9 +417,9 @@ class RemotesParser(object):
 
         if self.__end_block(name):
             if 'remote' == name:
-                remote = Remote(self.__filename, self.__vendor, self.__product,
-                                self.__properties, dict(self.__key_codes),
-                                self.__contributor)
+                remote = Remote(self.__database, self.__filename,
+                                self.__vendor, self.__product, self.__contributor,
+                                self.__properties, dict(self.__key_codes))
 
                 self.__remotes.append(remote)
                 self.__next_remote()
@@ -419,7 +481,7 @@ class RemotesParser(object):
 
         return False
 
-    def read(self, reader, filename=None):
+    def read(self, reader, filename=None, database=None):
         '''
         Parses the configuration found in reader.
         '''
@@ -430,6 +492,8 @@ class RemotesParser(object):
             filename = getattr(reader, 'name', str(reader.__class__))
 
         self.__filename = filename
+        self.__database = database
+
         self.__current_line = 0
 
         for line in reader:
@@ -452,6 +516,15 @@ class RemotesParser(object):
     current_block = property(lambda self: len(self.__blocks) and self.__blocks[-1])
     toplevel_block = property(lambda self: len(self.__blocks) and self.__blocks[0])
 
+class KeyCodeCategory(object):
+    '''
+    Container for various key-code categories.
+    '''
+
+    DEFAULT = _('Default Namespace')
+    CUSTOM = _('Custom Key Code')
+    ELISA = _('Elisa Compatible')
+
 def create_commands_table():
     '''
     Create a dict mapping the command name to full details,
@@ -463,7 +536,7 @@ def create_commands_table():
 
     #TODO: Add other keys (and alternatives key names):
     commands_by_category = [
-        (_('Default Namespace'), (
+        (KeyCodeCategory.DEFAULT, (
             ('BTN_LEFT',           'move_up_key',        _('Move Up')),
             ('BTN_MODE',           'move_up_key',        _('Move Up')),
             ('BTN_RIGHT',          'move_up_key',        _('Move Up')),
@@ -584,7 +657,7 @@ def create_commands_table():
         )),
 
         # Key names used by Elisa (as of 2007-02-12):
-        (_('Elisa Compatible'), (
+        (KeyCodeCategory.ELISA, (
             ('GO_UP',              'move_up_key',              _('Move Up')),
             ('GO_DOWN',            'move_down_key',            _('Move Down')),
             ('GO_LEFT',            'move_left_key',            _('Move Left')),
@@ -614,7 +687,7 @@ def create_commands_table():
         # standard names, and we should probably warn about (or ignore as
         # broken) non-standard key names, because applications are unlikely
         # to work with those broken lircd.conf files.
-        (_('Custom Key Code'), (
+        (KeyCodeCategory.CUSTOM, (
             ('CH_UP',              'ch_up_key',              _('Channel Up')),
             ('CH_DOWN',            'ch_down_key',            _('Channel Down')),
             ('<<',                 'rewind_key',             _('Rewind')),
@@ -724,6 +797,7 @@ class KeyListener(gobject.GObject):
         self.__reconnect_source = 0
         self.__socket_source = 0
         self.__socket = None
+        self.__buffer = ''
 
     def start(self):
         '''
@@ -738,8 +812,12 @@ class KeyListener(gobject.GObject):
         Connects to the lircd socket.
         '''
 
-        logging.info('Trying to connect to lircd...')
+        self.__disconnect()
+
+        logging.info('trying to connect to lircd...')
+
         self.__socket = None
+        self.__buffer = ''
 
         try:
             # pylint: disable-msg=W0613
@@ -748,33 +826,54 @@ class KeyListener(gobject.GObject):
                 Handle I/O events on the lircd socket.
                 '''
 
+                logging.info('I/O event on lirc socket %d: %d', fd, condition)
+
                 if condition & gobject.IO_IN:
-                    packet = self.__socket.recv(128)
+                    logging.info('reading from lirc socket %d...', fd)
+                    packet = self.__socket and self.__socket.recv(128)
 
-                    if not packet:
-                        self.__reconnect()
-                        return False
+                    logging.info('...%d bytes received.', len(packet))
+                    self.__buffer += packet
 
-                    try:
-                        code, repeat, name, remote = packet.split()
-                        repeat = int(repeat, 16)
-                        code = long(code, 16)
+                    while True:
+                        eol = self.__buffer.find('\n')
 
-                        # pylint: disable-msg=E1101
-                        self.emit('key-pressed', remote, repeat, name, code)
+                        if eol < 0:
+                            break
 
-                    # pylint: disable-msg=W0704
-                    except ValueError:
-                        pass
+                        packet = self.__buffer[:eol]
+                        self.__buffer = self.__buffer[eol + 1:]
+
+                        logging.info('processing packet: %r', packet)
+
+                        try:
+                            code, repeat, name, remote = packet.split()
+                            repeat = int(repeat, 16)
+                            code = long(code, 16)
+
+                            # pylint: disable-msg=E1101
+                            self.emit('key-pressed', remote, repeat, name, code)
+
+                        # pylint: disable-msg=W0704
+                        except ValueError:
+                            pass
+
+                if condition & gobject.IO_HUP:
+                    self.__disconnect()
+                    return False
 
                 return True
 
             self.__socket = socket(AF_UNIX)
             self.__socket_source = gobject.io_add_watch(self.__socket.fileno(),
-                                                        gobject.IO_IN, on_io)
+                                                        gobject.IO_IN | gobject.IO_HUP,
+                                                        on_io)
 
             self.__socket.connect(self.__socket_name)
-            logging.info('Listening on %s...', self.__socket_name)
+
+            logging.info('listening on %s (fd=%d, source=%d)...',
+                         self.__socket_name, self.__socket.fileno(),
+                         self.__socket_source)
 
             # pylint: disable-msg=E1101
             self.emit('changed')
@@ -783,6 +882,7 @@ class KeyListener(gobject.GObject):
 
         except SocketError, ex:
             if ex[0] not in (errno.ENOENT, errno.ECONNREFUSED):
+                # pychecker says "Object (ex) has no attribute (message)", and that's not true.
                 logging.error('Cannot connect to %s: %s', self.__socket_name, ex.message)
 
             self.__disconnect()
@@ -793,6 +893,8 @@ class KeyListener(gobject.GObject):
         Disconnects from the lircd socket.
         '''
 
+        self.__buffer = ''
+
         if self.__socket_source:
             logging.info('disconnecting source: %d', self.__socket_source)
             gobject.source_remove(self.__socket_source)
@@ -811,8 +913,6 @@ class KeyListener(gobject.GObject):
         Tries to reconnects to the lircd socket.
         '''
 
-        self.__disconnect()
-
         if not self.__reconnect_source:
             self.__reconnect_source = gobject.timeout_add(5000, self._on_reconnect)
 
@@ -830,7 +930,7 @@ class KeyListener(gobject.GObject):
     # pylint: disable-msg=W0212,W1001
     connected = property(lambda self: None != self.__socket)
 
-class HardwareConfParser:
+class HardwareConfParser(object):
     '''
     Parse the (Debian/Ubuntu-specific) /etc/lirc/hardware.conf file.
 
@@ -877,3 +977,19 @@ class HardwareConfParser:
         except KeyError:
             return default
 
+def has_include_keyword():
+    '''Checks if include statements are supported in lircd.conf.'''
+
+    return (config.LSB_RELEASE.check(name='Ubuntu', codename='hardy') or
+            config.LSB_RELEASE.check(name='Ubuntu', release='8.04'))
+
+def find_remote_config():
+    '''Finds the location our customized lircd.conf file.'''
+
+    if has_include_keyword():
+        return os.path.join(config.LIRC_CONFDIR, 'lircd.conf.gnome'), True
+
+    return os.path.join(config.LIRC_CONFDIR, 'lircd.conf'), False
+
+if '__main__' == __name__:
+    print find_remote_config()
